<?php

namespace Drupal\hospital_price_transparency\Form;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\hospital_price_transparency\HospitalPriceTransparencyService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configure Hospital Price Transparency settings for this site.
 */
class HospitalPriceTransparencyForm extends ConfigFormBase {

  /**
   * Hospital Price Transparency service.
   *
   * @var \Drupal\hospital_price_transparency\HospitalPriceTransparencyService
   */
  protected $hospitalPriceTransparency;

  /**
   * Entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructor.
   *
   * @param \Drupal\hospital_price_transparency\HospitalPriceTransparencyService $hospital_price_transparency
   *   Hospital Price Transparency service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity type manager.
   */
  public function __construct(HospitalPriceTransparencyService $hospital_price_transparency, EntityTypeManagerInterface $entity_type_manager) {
    $this->hospitalPriceTransparency = $hospital_price_transparency;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('hospital_price_transparency'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'hospital_price_transparency_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['hospital_price_transparency.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $existing_file_url = $this->hospitalPriceTransparency->getFileUrl();
    $form['info'] = [
      '#type' => 'html_tag',
      '#tag' => 'p',
      '#value' => $this->t('The Hospital Price Transparency module is intended to make it easier for site builders
        to meet the requirements outlined by <a href="https://www.cms.gov/files/document/steps-machine-readable-file.pdf"
        target="_blank">cms.gov</a> for providing a comprehensive machine-readable file of all items and services, in
        particular Steps 5 and 6 of that outline. However, the maintainers of this module do not claim implicitly or
        explicitly that using this module fulfills all or any legal obligations of any hospital.'),
    ];

    $form['path_fields'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Path to file'),
      '#tree' => FALSE,
    ];
    $form['path_fields']['info'] = [
      '#type' => 'html_tag',
      '#tag' => 'p',
      '#value' => $this->t('Per Step 6 of the <a href="https://www.cms.gov/files/document/steps-machine-readable-file.pdf"
        target="_blank">cms.gov guidelines</a>, we are told that the file should (a) be accessible to automated searches and
        (b) be accessible to direct file downloads through a link. To that end, The file will always be available at
        <a href="/hospital-price-transparency" target="_blank">/hospital-price-transparency</a>. In fact, a link to that
        route was automatically added to the footer menu when you enabled this module, though you may prefer disabling
        that link and making your own. You should also add that path to your sitemap. This static path is more
        dependable than using the file url directly.'),
    ];
    $form['path_fields']['path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Additional human-friendly path to file'),
      '#default_value' => $this->config('hospital_price_transparency.settings')->get('path'),
      '#description' => $this->t('You may optionally expose the file at a second human-friendly path with this
         field. Begin the path with a forward slash. Example: <code>/pricing</code>'),
      '#required' => FALSE,
    ];

    $form['hospital_fields'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Hospital Information'),
      '#tree' => FALSE,
    ];
    $form['hospital_fields']['info'] = [
      '#type' => 'html_tag',
      '#tag' => 'p',
      '#value' => $this->t('Per Step 5 of the <a href="https://www.cms.gov/files/document/steps-machine-readable-file.pdf"
        target="_blank">cms.gov guidelines</a> any file you upload will be renamed using the pattern
        <nobr><code>[ein]_[hospitalname]_standardcharges.[extension]</code></nobr>.'),
    ];
    if ($existing_file_url) {
      $form['hospital_fields']['info_rename'] = [
        '#type' => 'html_tag',
        '#tag' => 'p',
        '#value' => $this->t('If you update these values, the <a href=":url" target="_blank">existing price transparency file</a>
           will automatically be renamed accordingly.', [':url' => $existing_file_url->toString()]),
      ];
    }

    $form['hospital_fields']['ein'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Employer Identification Number (EIN)'),
      '#default_value' => $this->config('hospital_price_transparency.settings')->get('ein'),
      '#description' => $this->t('Enter the nine-digit EIN with no hyphen'),
      '#pattern' => '[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]',
      '#required' => TRUE,
    ];
    $form['hospital_fields']['hospital_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Hospital Name'),
      '#default_value' => $this->config('hospital_price_transparency.settings')->get('hospital_name'),
      '#required' => TRUE,
    ];

    $form['file_fields'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('File Upload'),
      '#tree' => FALSE,
    ];
    if ($existing_file_url) {
      $form['file_fields']['info'] = [
        '#type' => 'html_tag',
        '#tag' => 'p',
        '#value' => $this->t('If you upload a file, the <a href=":url" target="_blank">current file</a> will be overwritten and/or deleted.', [':url' => $existing_file_url->toString()]),
      ];
    }
    $form['file_fields']['file'] = [
      '#type' => 'managed_file',
      '#title' => $this->t('Hospital Price Transparency file'),
      '#description' => $this->t('This must be a machine-readable <code>xml</code>, <code>csv</code>, or <code>json</code> file.'),
      '#upload_validators'  => [
        'file_validate_extensions' => ['xml csv json'],
      ],
      '#required' => FALSE,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $path = $form_state->getValue('path');
    if ($path && !str_starts_with($path, "/")) {
      $form_state->setErrorByName('path', $this->t('The path must start with a forward slash'));
    }
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('hospital_price_transparency.settings')
      ->set('ein', $form_state->getValue('ein'))
      ->set('hospital_name', $form_state->getValue('hospital_name'))
      ->set('path', $form_state->getValue('path'))
      ->save();

    $fid = $form_state->getValue('file');
    if (isset($fid[0])) {
      $file = $this->entityTypeManager->getStorage('file')->load($fid[0]);
      if ($file !== NULL) {
        $this->hospitalPriceTransparency->setPriceTransparencyFile($file);
        $file->delete();
      }
    }
    else {
      $this->hospitalPriceTransparency->updateFilename();
    }

    parent::submitForm($form, $form_state);
  }

}
