<?php

namespace Drupal\hospital_price_transparency;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\file\FileInterface;

/**
 * Service for hospital_price_transparency module.
 */
class HospitalPriceTransparencyService {

  use StringTranslationTrait;

  /**
   * Config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The file url generator service.
   *
   * @var \Drupal\Core\File\FileUrlGeneratorInterface
   */
  protected $fileUrlGenerator;

  /**
   * The messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   Config factory.
   * @param \Drupal\Core\State\StateInterface $state
   *   State.
   * @param \Drupal\Core\File\FileUrlGeneratorInterface $file_url_generator
   *   Url generator.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   Messenger.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system.
   */
  public function __construct(ConfigFactoryInterface $config_factory, StateInterface $state, FileUrlGeneratorInterface $file_url_generator, MessengerInterface $messenger, FileSystemInterface $file_system) {
    $this->configFactory = $config_factory;
    $this->state = $state;
    $this->fileUrlGenerator = $file_url_generator;
    $this->messenger = $messenger;
    $this->fileSystem = $file_system;
  }

  /**
   * Get the url to the current file.
   */
  public function getFileUrl(): Url|NULL {
    $current_filename = $this->state->get('hospital_price_transparency.filename');
    if ($current_filename) {
      return $this->fileUrlGenerator->generate('public://' . $current_filename);
    }
    return NULL;
  }

  /**
   * Copy file from a File entity and use it as the price transparency file.
   *
   * This also deletes any existing price transparency file.
   *
   * @param \Drupal\file\FileInterface $file
   *   File entity whose file should be copied and used as transparency file.
   */
  public function setPriceTransparencyFile(FileInterface $file): void {
    $extension = pathinfo($file->getFileUri(), PATHINFO_EXTENSION);
    // The form has validation for the extension, but we validate here as well,
    // in case the API is being used independent of the form.
    if (!in_array($extension, ['xml', 'json', 'csv'], TRUE)) {
      throw new \InvalidArgumentException("File type '$extension' was not one of allowed types xml, json, or csv");
    }
    $new_filename = $this->buildFilename() . '.' . $extension;
    // Per cms.gov guidelines, this must be a public file.
    $this->fileSystem->copy($file->getFileUri(), 'public://' . $new_filename, FileSystemInterface::EXISTS_REPLACE);
    $current_filename = $this->state->get('hospital_price_transparency.filename');
    if ($current_filename) {
      if ($current_filename !== $new_filename) {
        $this->fileSystem->delete('public://' . $current_filename);
        $this->state->set('hospital_price_transparency.filename', $new_filename);
        $this->messenger->addStatus($this->t("@current_filename was replaced by @new_filename", [
          '@current_filename' => $current_filename,
          '@new_filename' => $new_filename,
        ]));
      }
      else {
        $this->messenger->addStatus($this->t("@current_filename was updated", [
          '@current_filename' => $current_filename,
        ]));
      }
    }
    else {
      $this->state->set('hospital_price_transparency.filename', $new_filename);
      $this->messenger->addStatus($this->t("@new_filename was created", [
        '@new_filename' => $new_filename,
      ]));
    }
  }

  /**
   * Deletes the file without replacement.
   */
  public function deletePriceTransparencyFile(): void {
    $current_filename = $this->state->get('hospital_price_transparency.filename');
    if ($current_filename) {
      $this->fileSystem->delete('public://' . $current_filename);
      $this->messenger->addStatus($this->t("@current_filename was deleted. There is no longer an active price transparency file.", [
        '@current_filename' => $current_filename,
      ]));
    }
    $this->state->delete('hospital_price_transparency.filename');
  }

  /**
   * Updates the filename of the existing price transparency file.
   *
   * This is necessary if the EIN or Hospital Name change.
   */
  public function updateFilename(): void {
    if ($this->getFileUrl()) {
      $current_filename = $this->state->get('hospital_price_transparency.filename');
      $current_extension = pathinfo($current_filename, PATHINFO_EXTENSION);
      $new_filename = $this->buildFilename() . '.' . $current_extension;
      if ($current_filename !== $new_filename) {
        $this->fileSystem->move('public://' . $current_filename, 'public://' . $new_filename, FileSystemInterface::EXISTS_REPLACE);
        $this->state->set('hospital_price_transparency.filename', $new_filename);
        $this->messenger->addStatus($this->t("@current_filename was replaced by @new_filename", [
          '@current_filename' => $current_filename,
          '@new_filename' => $new_filename,
        ]));
      }
    }
  }

  /**
   * Builds standard filename per Step 5 of documentation, without extension.
   *
   * @see https://www.cms.gov/files/document/steps-machine-readable-file.pdf
   *
   * @return string
   *   The filename without the extension.
   */
  protected function buildFilename(): string {
    // We can't load the config in the constructor because the settings form
    // loads the service, then updates config, then uses the service. We need
    // the service to have access to the very latest config at all times.
    $config = $this->configFactory->get('hospital_price_transparency.settings');
    $parts = [
      $config->get('ein'),
      $config->get('hospital_name'),
      'standardcharges',
    ];
    return implode("_", $parts);
  }

}
